#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Stuart Poulin Dango of IBM Rational Software.
# 
#
# Updated by: Stuart Poulin
# Updated on: 20081117
# Design by Dave Bellagio, Bob Myers, Stu Poulin
######################################################################################################################
# 
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user		The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname		The schema repository connection to the database
#	dbset		The user database connection
#	baseline	The name of the baseline record
#	pvob		The name of the project vob for this baseline
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use LWP::Simple;
use POSIX;

use get_record_details;
use create_record;
my $promotion_level;
my $slashvob;
my $noslashvob;
my ($entity);
my ($failure);
my ($status,$resultSetStatus); #thl2
use subs qw(get_promotionlevel main);
# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $progname = basename $0;

my $result = main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

#####
# Subroutine to display the usage
sub Usage {
    print "

    Analyze Baseline Records in ALM
    Usage:
        $progname -user <username>
         -pw <password>
         -dbname <database name>
         -dbset <database set>
         -pvob <pvob>
         -baseline <baseline> 
		 -logfile <logfile>
		 -test
		 -help
";
return 0;
}

#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s		
    pw=s		
    dbname=s		
    dbset=s		
    pvob=s		
    baseline=s	
    logfile=s
    test
    help
    in=s
    out=s
);

my %opts = (
    user => $ENV{ALMUSER},
    pw => $ENV{ALMPW},
    dbname => $ENV{ALMDBNAME},
    dbset => $ENV{ALMDBSET},
    pvob => undef,
    baseline => undef,
    logfile => undef,
    in=>undef,
    out=>undef,
);


####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
     	$logger->log_print_message("Printing Usage to the command line:");	
      	my $retval = &Usage;
      	return 0;
}

$opts{pw} = "" if ($opts{pw} eq q{""});

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "Could not create the log file directory: $logdir");
            		$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "Could not open the log file: $opts{logfile}.");
        	$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("Logging started at $log_time"); 
}	 



####
# Display the help usage
Usage if ($opts{help}); 

# Display test inputs to log file 
if ($opts{test}){
	$logger->log_print_message("main: Defined user inputs:");
	$logger->log_print_message("main: Defined user login is $opts{user}.");
	$logger->log_print_message("main: Defined dbset is $opts{dbset}.");
	$logger->log_print_message("main: Defined database name is $opts{dbname}.");
	$logger->log_print_message("main: Defined baseline name is $opts{baseline}.");
	$logger->log_print_message("main: Defined Project VOB is $opts{pvob}.");
	if ($opts{logfile}){ $logger->log_print_message("Defined logfile is $opts{logfile}.");}
	return 1;
}

####
# End program if any of the parameters are not defined
my $msg = "";
$msg .= "Missing required options: -user username\n" if !defined $opts{user};
$msg .= "Missing required options: -pw password\n" if !defined $opts{pw};
$msg .= "Missing required options: -dbname dbname\n" if !defined $opts{dbname};
$msg .= "Missing required options: -dbset dbset\n" if !defined $opts{dbset};
$msg .= "Missing required options: -pvob pvob\n" if !defined $opts{pvob};
$msg .= "Missing required options: -baseline baseline\n" if !defined $opts{baseline};
if ($msg) {
  $logger->log_print_message("$msg");
  my $retval = &Usage();
  return 0;
}
;


####
#### Start a CQ session
my $session = CQPerlExt::CQSession_Build();
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
#### Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
	$logger->log_print_error(__LINE__ . "main: Login to $opts{dbname} failed: $@"); 
	return 0;
}

$logger->log_print_success("main: Login to $opts{dbname} succeeded.");
#

($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});


# 
# Get last X baselines from passed in file.
#
open(BL,"<$opts{in}") or die "Can't open $opts{in} for input : $!";
# history.action_timestamp, Name, PVOB_OrLocation, Project.Name, Project.Category, Project.Release, Project.id, UCM_Stream
my @previous_bl;
my $bl_count;
while (<BL>) {
	next if /^\s*#.*/; #commented header
	#last if $bl_count++ > 5;
	my ($date,$name,$pvob,$proj,$cat,$rel,$proj_id,$stream) = split /,/;
	push @previous_bl,[$name,$pvob];
}
close(BL);

# For each baseline, look up to see if it's part of a "Composed_of_Baselines", 
# if so, then look up the project of the parent baseline
my %projects_with_bls_in_composite;
#
    for my $bl (@previous_bl) {
	my $blcq = new get_record_details(session=>$session, type=>'ALMBaseline',fields=>{Name=>$bl->[0], PVOB_OrLocation=>$bl->[1]},top=>1);
	print "Examining Baseline: $bl->[0] $bl->[1]\n";
	#TODO Get last "promoted baseline" prior to this one.;
	#print "	Baselines in BaselinesInComposites\n";
	for my $bl_c ($blcq->get('BaselinesInComposites')) {
		last if ! $bl_c;
		# get Baseline details
		# returns: pvob%20%baselinename;
		my ($pvob,$baseline) = split /\s+/,$bl_c,2;
		#print "PVOB = $pvob, Baseline = $baseline\n";
		# Get this baseline and find the project.
		my $blParent = new get_record_details(session=>$session, type=>'ALMBaseline',fields=>{Name=>$baseline, PVOB_OrLocation=>$pvob},top=>1);
		my ($project) = $blParent->get('Project');
		#print "Project = $project\n";
		# Save it in a hash
		$projects_with_bls_in_composite{$project}++;

			
	}
    }
    print "\n";
    print "Projects usings baselines \n";
    for my $project (keys %projects_with_bls_in_composite ) {
	    print "  project: $project\n";
		# Get tasks of type "Integrate Baseline" from this project.
		# For now assume there's only one such task.
		# Create an activity on this task of "Integration Baseline"
	    my $task = new get_record_details(session=>$session, type=>'ALMTask',fields=>{Type=>"Integrate Baseline", Project=>$project, State=>"Opened"},top=>1,);
	    if ($task) {
		    my ($task_id) = $task->get('id');
		    my ($task_headline) = $task->get('Headline');
		    print "    Task id: $task_id    headline: $task_headline\n";
		    my $activity = new create_record(
		        session=>$session,
		        type=>'ALMActivity',
		        fields => [
			"Task"=>$task_id,
			"Headline" => "Integrate Baseline:  baseline $opts{baseline} : pvob $opts{pvob}",
			"Type"=>"Integrate Baseline",
			"Owner"=>"ReleaseEngineer_ToDo",
		      ],

	               );
	            #TODO Error checking.
	            $activity->Validate;
	            $activity->Commit;    
		    print "    New Activity : ",$activity->GetDisplayName();
		    print "\n";
	    }
	    else {
		    print qq{     No task of type "Integrate Baseline" in project\n};
	    }


    }
1;
} #end main

__END__

